import 'package:test/test.dart';
import 'package:wallet_utils/src/validate.dart';

main() {
  group('name', () {
    test('invalidate bad entire asset name', () {
      expect(isAssetNameGood(r'abcefghijklmnopqrstuvwxyzabcefghi'), false);
      expect(isAssetNameGood(r'a'), false);
      expect(isAssetNameGood(r'abc'), false);
      expect(isAssetNameGood(r'RVN'), false);
      expect(isAssetNameGood(r'$123!'), false);
      expect(isAssetNameGood(r'$ABC!'), false);
      expect(isAssetNameGood(r'ABC/'), false);
      expect(isAssetNameGood(r'$ABC/'), false);
      expect(isAssetNameGood(r'$ABC/abc'), false);
      expect(isAssetNameGood(r'$ABC/ABC'), false);
      expect(isAssetNameGood(r'#ABC/ABC'), false);
      expect(isAssetNameGood(r'ABC!/ABC'), false);
      expect(isAssetNameGood(r'#ABC/#ABC#ABC'), false);
      expect(isAssetNameGood(r'#ABC/~ABC'), false);
      expect(isAssetNameGood(r'ABC._'), false);
      expect(isAssetNameGood(r'ABC._ABC'), false);
      expect(isAssetNameGood('RV'), false);
      expect(isAssetNameGood('RVN'), false);
      expect(isAssetNameGood('RAVEN'), false);
      expect(isAssetNameGood('RAVENCOIN'), false);
      expect(isAssetNameGood('RavenCoin5000'), false);
      expect(isAssetNameGood('THIS_IS_THIRTY_TWO_XX_CHARS_LONG'), false);
      expect(isAssetNameGood('THIS_ONE_is_lower_case'), false);
      expect(isAssetNameGood('THIS_ONE HAS SPACES'), false);
      expect(isAssetNameGood('THIS_ONE.HAS.DOUBLE..PERIODS'), false);
      expect(isAssetNameGood('THIS_ONE.HAS._DOUDLBE_PUNC'), false);
      expect(isAssetNameGood('THIS_ONE.HAS_.DOUDLBE_PUNC'), false);
      expect(isAssetNameGood('THIS_ONE.HAS__DOUDLBE_PUNC'), false);
      expect(isAssetNameGood('THIS_ONE_ENDS_WITH_PERIOD.'), false);
      expect(isAssetNameGood('THIS_ONE_ENDS_WITH_'), false);
      expect(isAssetNameGood('.THIS_ONE_STARTS_WITH_PERIOD'), false);
      expect(isAssetNameGood('_THIS_ONE_STARTS_WITH_US'), false);
      expect(isAssetNameGood('THIS_CONTAIN_&_BAD_PUNC'), false);
      expect(isAssetNameGood(''), false);
    });
    test('validate good entire asset name', () {
      expect(isAssetNameGood(r'123!'), true);
      expect(isAssetNameGood(r'$123'), true);
      expect(isAssetNameGood(r'#123'), true);
      expect(isAssetNameGood(r'123'), true);
      expect(isAssetNameGood(r'ABC'), true);
      expect(isAssetNameGood(r'$ABC'), true);
      expect(isAssetNameGood(r'ABC!'), true);
      expect(isAssetNameGood(r'#ABC'), true);
      expect(isAssetNameGood(r'ABC/ABC'), true);
      expect(isAssetNameGood(r'ABC/ABC/ABC'), true);
      expect(isAssetNameGood(r'ABC/ABC!'), true);
      expect(isAssetNameGood(r'ABC/ABC/ABC!'), true);
      expect(isAssetNameGood(r'ABC/ABC/ABC~ABC'), true);
      expect(isAssetNameGood(r'ABC/ABC/ABC#ABC'), true);
      expect(isAssetNameGood(r'ABC/ABC#ABC'), true);
      expect(isAssetNameGood(r'ABC#ABC'), true);
      expect(isAssetNameGood(r'ABC/ABC~ABC'), true);
      expect(isAssetNameGood(r'ABC~ABC'), true);
      expect(isAssetNameGood(r'#ABC/#ABC'), true);
      expect(isAssetNameGood(r'#ABC/#ABC/#ABC'), true);
      expect(isAssetNameGood('THIS_IS_THIRTY_XXXX_CHARS_LONG'), true);
      expect(isAssetNameGood('THIS_IS_THIRTY_ONEX_CHARS_LONG!'), true);
      expect(isAssetNameGood('THIS_IS_NOT_THIRTY_CHARS_LONG'), true);
      expect(isAssetNameGood('THIS_IS_NOT_THIRTY_CHARS_LONG!'), true);
      expect(isAssetNameGood('THIS_ONE.HAS.PERIODS'), true);
      expect(isAssetNameGood('RAVENCOIN5000'), true);
      expect(isAssetNameGood('FANFT/RAVENHEAD24#PaintedRVN5'), true);
    });
    test('testing names from ravencoin.asset-explorer.net', () {
      expect(isAssetNameGood(r'HIGH_RAVEN_SOCIETY/S1#771'), true);
      expect(isAssetNameGood(r'CYBER_SAMURAI/HOLO_GRAPHIC#87'), true);
      expect(isAssetNameGood(r'ALPHABEAT/HIGHMOON#24'), true);
      expect(isAssetNameGood(r'MORGANCROW/CAKECLUB#5'), true);
      expect(isAssetNameGood(r'ANIMALXXING#440'), true);
      expect(isAssetNameGood(r'RAVENMINT/BUNNY_CLUB#217'), true);
      expect(isAssetNameGood(r'XPLNFT#RVNVRS_The_River'), true);
      expect(isAssetNameGood(r'MFERS#19'), true);
      expect(isAssetNameGood(r'BILLION_APE_CLUB#02375'), true);
      expect(isAssetNameGood(r'WHITEHASH/RAVEN_FOLKS#52'), true);
      expect(isAssetNameGood(r'TUFFERTOON/TOOTLES#121-Accepta'), true);
      expect(isAssetNameGood(r'HIGH_RAVEN_SOCIETY/S1#770'), true);
      expect(isAssetNameGood(r'MFERS#10'), true);
      expect(isAssetNameGood(r'MORGANCROW'), true);
      expect(isAssetNameGood(r'WHITEHASH/RAVEN_FOLKS#43'), true);
      expect(isAssetNameGood(r'RAVENIST#SACK_FoundingFather1'), true);
      expect(isAssetNameGood(r'RAVENIST#GHOUL_Vision1'), true);
      expect(isAssetNameGood(r'KAIO/HOKU'), true);
      expect(isAssetNameGood(r'RAVENFLOWERS'), true);
      expect(isAssetNameGood(r'FRIZZY#A.I.96'), true);
      expect(isAssetNameGood(r'THE_PEOPLE#Person_165'), true);
      expect(isAssetNameGood(r'RAVENIST#SACKS_1'), true);
      expect(isAssetNameGood(r'RVNFT#JOAQUIN_LEVIATHAN'), true);
      expect(isAssetNameGood(r'WHITEHASH/RAVEN_FOLKS#42'), true);
      expect(isAssetNameGood(r'SYNTH#LAB_POC'), true);
      expect(isAssetNameGood(r'RAVENIST#eduART_Hornbilly'), true);
      expect(isAssetNameGood(r'RAVENIST#eduART_OilLamp'), true);
      expect(isAssetNameGood(r'RAVENIST#eduART_Hornbill'), true);
      expect(isAssetNameGood(r'RAVENIST#MAESTRO_Censure'), true);
      expect(isAssetNameGood(r'WHITEHASH#AI_Expo-PalmBeach'), true);
      expect(isAssetNameGood(r'ANIMALXXING#439'), true);
      expect(isAssetNameGood(r'ANIMALXXING#438'), true);
      expect(isAssetNameGood(r'COVIDPERP#KLAIN'), true);
      expect(isAssetNameGood(r'COVIDPERP#GARLAND'), true);
      expect(isAssetNameGood(r'ANIMALXXING#437'), true);
      expect(isAssetNameGood(r'THE_PEOPLE#Person_164'), true);
      expect(isAssetNameGood(r'ANIMALXXING#431'), true);
      expect(isAssetNameGood(r'THE_PEOPLE#Person_163'), true);
      expect(isAssetNameGood(r'MPHM/BEACHLOVERSMOON'), true);
      expect(isAssetNameGood(r'ALPHABEAT/HIGHMOON'), true);
      expect(isAssetNameGood(r'WHITEHASH/RAVEN_FOLKS#41'), true);
      expect(isAssetNameGood(r'ANIMALXXING#430'), true);
      expect(isAssetNameGood(r'MORGANCROW/WITHLOVE#BOAT'), true);
      expect(isAssetNameGood(r'MORGANCROW'), true);
      expect(isAssetNameGood(r'CYBER_SAMURAI/TAKO_YARO#40'), true);
      expect(isAssetNameGood(r'WHITEHASH/RAVEN_FOLKS#40'), true);
      expect(isAssetNameGood(r'CRYPTOHOMIES/V3#65'), true);
      expect(isAssetNameGood(r'WHITEHASH/RAVEN_FOLKS#39'), true);
      expect(isAssetNameGood(r'WISEORACLE#PAGE_OF_WANDS'), true);
      expect(isAssetNameGood(r'WHITEHASH#AI_EXPO-Abandons2'), true);
      expect(isAssetNameGood(r'WHITEHASH#AI_EXPO-RVN_Castle3'), true);
      expect(isAssetNameGood(r'WHITEHASH#AI_EXPO-Crows2'), true);
      expect(isAssetNameGood(r'WHITEHASH#AI_EXPO-DesertWander'), true);
      expect(isAssetNameGood(r'SIRCRONE#CRONENBERG22'), true);
      expect(isAssetNameGood(r'BVRN#OUTLOOK'), true);
      expect(isAssetNameGood(r'RAVENMINT/COOL_OWL#351'), true);
      expect(isAssetNameGood(r'ROYAL_BEAR_HIDEOUT/MINT#83'), true);
      expect(isAssetNameGood(r'COVIDPERP#HAINES.AVRIL_US'), true);
      expect(isAssetNameGood(r'COVIDPERP#BLINKEN'), true);
      expect(isAssetNameGood(r'COVIDPERP#NEUBERGER.ANNE_US'), true);
      expect(isAssetNameGood(r'COVIDPERP#MAYORKAS'), true);
      expect(isAssetNameGood(r'COVIDPERP#SLAVITT.ANDY_US'), true);
      expect(isAssetNameGood(r'COVIDPERP#ZIENTS.JEFFREY_US'), true);
      expect(isAssetNameGood(r'XPLNFT#RVNVRS_Magic_Beach'), true);
      expect(isAssetNameGood(r'THE_PEOPLE#Person_162'), true);
      expect(isAssetNameGood(r'TESSANICOLAI#BOAT'), true);
      expect(isAssetNameGood(r'RAVENIST#HG_Hobgoblin407'), true);
      expect(isAssetNameGood(r'RAVENIST#DEFI_Xanzibar'), true);
      expect(isAssetNameGood(r'TESSANICOLAI#BLUE_BOAT'), true);
      expect(isAssetNameGood(r'TESSANICOLAI#LOS_MUERTOS'), true);
      expect(isAssetNameGood(r'THE_PEOPLE#Person_161'), true);
      expect(isAssetNameGood(r'TESSANICOLAI#PIXEL_CHICKEN'), true);
      expect(isAssetNameGood(r'RAVENIST#MAESTRO_Blitz'), true);
      expect(isAssetNameGood(r'WHITEHASH/RAVEN_FOLKS#38'), true);
      expect(isAssetNameGood(r'FRIZZY#A.I.81'), true);
      expect(isAssetNameGood(r'FRIZZY#A.I.76'), true);
      expect(isAssetNameGood(r'RVNFT#JOAQUIN_RV9_FALCON_3D_5'), true);
      expect(isAssetNameGood(r'VINSENT_WINE'), true);
      expect(isAssetNameGood(r'VINSENT_WINE/385'), true);
      expect(isAssetNameGood(r'BVRN#MOAB'), true);
      expect(isAssetNameGood(r'BVRN#GLEAM'), true);
      expect(isAssetNameGood(r'BVRN#DICHOTOMY'), true);
      expect(isAssetNameGood(r'WHITEHASH/GO_FK_YOURSELF'), true);
      expect(isAssetNameGood(r'MFERS#9'), true);
      expect(isAssetNameGood(r'MPHM#BeachLoverMoon8of60'), true);
      expect(isAssetNameGood(r'MORGANCROW/CAKECLUB!'), true);
      expect(isAssetNameGood(r'XPLNFT!'), true);
    });
  });
}
